--[[ **************************************************************
Script to place the driver in a location that the OS will automatically find it and not show the user
the hardware wizard.

bContinueInstall is a global flag that is uses to let the calling app know if the action succeeded of failed.

KCF: 3-7-2012 Added the following to this script:
1) Check that the spooler service is running and if it isn't start it.  Based on Sangeeta's testing of field issue, she noted
	that the SP driver installation can sometimes crash the spooler and if it is not running our driver installation fails.
2) Run pnputil if the OS has it installed.  Added to workaround field issue where SP driver corrupts the driver store. 
	Sangeeta's testing showed running pnputil restored the driver store to a condition where dpinst can stage our driver
	correctly.
***********************************************************************]]--
local pathCommonAppDataFolder = Shell.GetFolder(SHF_APPLICATIONDATA_COMMON);
local pathTempDpinstFolder = pathCommonAppDataFolder.."\\Datacard\\XPS Card Printer";
bContinueInstall = true;

fDebug("\r\n\Staging driver...");

-- if the driver is signed using the Datacard Verisign certificate we cannot do a silent install
-- the user must be asked if they trust the certificate or the driver files will not stage correctly.
if (UseDCGCert) then
	fDebug("INFO: Datacard signed. Staging will ask user to trust our install.");
	strQuiet = "";
else
	strQuiet = " /Q";
end	

--[[ *********************************************************************
**************************** START ***************************************
**************************************************************************
KCF: Added 3-7-2012 for v3.0 release 
Check if the spooler service is running and if it isn't attempt to start it.
***********************************************************************]]--
fDebug("INFO: Checking if spooler service is running...");	
local ServiceToQueryDisplay = "Print Spooler";
local ServiceToQueryKey = "Spooler";
local LastError = 0;
local ArgWorkingDir = "";
local bWow64Disabled = false;

QueryResult = Service.Query(ServiceToQueryDisplay, ServiceToQueryKey);
if not (QueryResult == SERVICE_RUNNING) then
    -- The service is not running.
    LastError = 0; -- Set LastError to 0 so it is in a known good state to start with.
    if (QueryResult == SERVICE_PAUSED) then
    	fDebug("INFO: Spooler service is paused. Attempting to restore to running state...");	
        Service.Continue(ServiceToQueryDisplay, ServiceToQueryKey); -- The service is paused, try continue
        LastError = Application.GetLastError(); -- determine if this succeeded or failed
    elseif (QueryResult == SERVICE_STOPPED) then
    	fDebug("INFO: Spooler service is stopped. Attempting to restart...");
        Service.Start(ServiceToQueryDisplay, ServiceToQueryKey, nil); -- The service is stopped, try start
        LastError = Application.GetLastError(); -- determine if this succeeded or failed
    else -- State being returned is neither running, stopped, nor paused. Bail out because we need the spooler running.
    	fDebug("ERROR: Spooler service is returning state: "..QueryResult..". Driver installation will terminate.");
    	bContinueInstall = false;
    end   
    if ((LastError ~= 0) or (bContinueInstall == false)) then 
        -- An error happened with continue or restart of spooler. Bail out because we need the spooler running.
        fDebug("ERROR: Failed to get spooler service into running state.  Driver installation will terminate.");
        bContinueInstall = false;
    else
    	-- The service was restored to running.  OK to continue with the installation.
    	bContinueInstall = true;
        fDebug("SUCCESS: The spooler service has be restored to a running state.");
    end
else
    -- The service is running
    bContinueInstall = true;
    fDebug("SUCCESS: The spooler service is running.");
end

--[[ ************************************************************************************
KCF: Added 3-7-2012 for v3.0 release
Next, we need to run pnputil if the OS has it available (Vista and above should)
If it cannot be found or is found and fails during the call we go on with the installation
because we still have a reasonable chance of success using just dpinst to stage the driver.
************************************************************************************]]--
if (bContinueInstall) then
	-- Need to determine if pnputil.exe is on this system. Doing this for all OS's as this is more
	-- straightforward than finding which OS's support the utility. Looking in the System Folder
	-- (Windows\System32 on most systems).
	fDebug("INFO: Checking if system has pnputil.exe...");
	local PnPutilFound = File.Find(_SystemFolder, "PnPutil.exe", true, true); -- Chose this function because PnPUtil.exe might be in subdirectoy of System32.
	if (PnPutilFound) then
		fDebug("SUCCESS: Found pnputil.exe at:"..PnPutilFound[1]);
		-- Need to determine if this is a USB or Network install and point to the correct folder holding the INF file(s).
		if (connectType == "USB") then
			ArgWorkingDir = pathTempDpinstFolder.."\\USB";
		else
			ArgWorkingDir = pathTempDpinstFolder.."\\network";
		end
		
		-- Next need to disable file system redirection for 64-bit OS or the Shell Execute cannot find pnputil.exe
		if (Is64bit and (not bWow64Disabled)) then
			fDebug("INFO: 64-bit OS detected. Temporarily disabling file system redirection.");
			DLL.CallFunction(_WindowsFolder.."\\SysWOW64\\kernel32.dll", "Wow64DisableWow64FsRedirection", "\""..Application.GetWndHandle().."\"", DLL_RETURN_TYPE_LONG, DLL_CALL_STDCALL);
			Application.Sleep(50); 
			bWow64Disabled = true;
		end
		
		fDebug("INFO: Running pnputil.exe against INF files at: "..ArgWorkingDir);
		-- Run pnputil.exe here. 
		-- NOTE: You must run from the directory containing the INF files or this will not work!!
		File.Run("pnputil.exe", "-a *.INF", ArgWorkingDir, SW_HIDE, true);
		Application.Sleep(100); 
		-- Check to see if an error occurred when launching the file.
    	LastError = Application.GetLastError();
    	-- If an error occurred, display an error message to the user.
    	if (LastError ~= 0) then
        	fDebug("ERROR: Call to run pnputil failed with error "..LastError..": ".._tblErrorMessages[LastError]);
    	end
		
		-- Need to reenable file system redirection so system acts as expected.
		if (Is64bit and bWow64Disabled) then
			fDebug("INFO: 64-bit OS detected. Re-enabling file system redirection.");
			DLL.CallFunction(_WindowsFolder.."\\SysWOW64\\kernel32.dll", "Wow64RevertWow64FsRedirection", "\""..Application.GetWndHandle().."\"" , DLL_RETURN_TYPE_LONG, DLL_CALL_STDCALL);
			Application.Sleep(50); 
			bWow64Disabled = false;
		end
		
	else -- pnputil.exe not found.
		LastError = Application.GetLastError();
		fDebug("INFO: Pnputil.exe not found at: ".._SystemFolder);
    	-- If an error occurred, display an error message to the user.
    	if (LastError ~= 0) then
        	fDebug("ERROR: Call to find pnputil failed with error "..LastError..": ".._tblErrorMessages[LastError]);
    	end
	end
end

-- Remove the following two lines once we have confidence that pnputil is working.
-- Dialog.Message("DEBUG", "Check pnputil outcome", MB_OK, MB_INFORMATION, MB_DEFBUTTON1);
-- bContinueInstall = false;

--[[ *****************************************************************
*******************  END OF 3-7-12 CHANGE  ***************************
**********************************************************************]]--

if (bContinueInstall) then -- Spooler is running and PnPutil run if found.  Time to stage the driver files.
	if (Is64bit) then
		if (IsXPLevel and IsServer) then
			File.Run(pathTempDpinstFolder.."\\dpinst64.exe", strQuiet, "", SW_SHOWNORMAL, true);
		else
			File.Run(pathTempDpinstFolder.."\\dpinst64.exe", "/LM"..strQuiet.." /SE /SA", "", SW_SHOWNORMAL, true);
		end
	else
		if (IsXPLevel and IsServer) then
			File.Run(pathTempDpinstFolder.."\\dpinst32.exe", strQuiet, "", SW_SHOWNORMAL, true);
		else
			File.Run(pathTempDpinstFolder.."\\dpinst32.exe", "/LM"..strQuiet.." /SE /SA", "", SW_SHOWNORMAL, true);
		end
	end	
	LastError = Application.GetLastError();	
	if (LastError == 0) then
		bContinueInstall = true;
		fDebug("SUCCESS: Driver files staged using dpinst.");
	else
		bContinueInstall = false;
		fDebug("ERROR: Driver files not staged. Call to dpinst failed with error: "..LastError..": ".._tblErrorMessages[LastError]);
	end	
end		