--[[
******************************************************************
Script to uninstall the magnetic stripe fonts on the system.

 ***** IMPORTANT NOTE *****
The script to uninstall fonts does not need to be called first. 
If the font is detected on the system it will be unregistered before
the new font file is copied and registered on the system.
******************************************************************
]]--
fDebug("\r\n\Removing magstripe fonts...");
-- Reset the return status string so it contains only text from this script when it returns
local bContinue = true;
local tblFileNames = { "DXP01_T1.ttf", "DXP01_T2.ttf","DXP01_T3.ttf","DXP01_JIS.ttf" };
local tblFontNames = { "Mag Track 1 XPS Card Printer", "Mag Track 2 XPS Card Printer", "Mag Track 3 XPS Card Printer", "Mag Track JIS XPS Card Printer"};

-- ************************************
--   Get Fonts folder path
-- ************************************
local fonts_dir = Shell.GetFolder(SHF_FONTS); -- Gets the path to the user's Font directory.
local error = Application.GetLastError();
if (error ~= 0) then
	fDebug("ERROR: Error detecting system font folder: "..error);
	bContinue = false;
end

if (bContinue) then
	local fontCount = Table.Count(tblFontNames);
	local loopCounter = 1;
	repeat  
		local strFontFileName = tblFileNames[loopCounter];
		local strFontName = tblFontNames[loopCounter];
		--  ************************************
		--  Detect if the font is installed.  If it is, unregister it.
		--  ************************************
		if (File.DoesExist(fonts_dir.."\\"..strFontFileName)) then -- Check if the font exits.  If it does, unregister it.
			fDebug("  Magstripe font detected: "..strFontFileName);
			System.UnregisterFont(fonts_dir.."\\"..strFontFileName, strFontName, true);
			error = Application.GetLastError();
			if (error ~= 0) then
				fDebug("  Error unregistering font: "..error);
			else
				fDebug("  Font unregistered.");
			end
		end
	loopCounter = loopCounter + 1;
	until loopCounter > fontCount
end	

-- Send a signal back to the main script using the bContinueInstall flag.
-- always continue. Don't abort the driver installation because of a font issue.
bContinueInstall = true;
fDebug("COMPLETE: Magstripe fonts removal.\r\n");

